@@ -51,7 +51,8 @@
 #define HIDPP_CAP_BUTTON_KEY_1b04			(1 << 2)
 #define HIDPP_CAP_BATTERY_LEVEL_1000			(1 << 3)
 #define HIDPP_CAP_KBD_REPROGRAMMABLE_KEYS_1b00		(1 << 4)
-#define HIDPP_CAP_ONBOARD_PROFILES_8100			(1 << 5)
+#define HIDPP_CAP_COLOR_LED_EFFECTS_8070			(1 << 5)
+#define HIDPP_CAP_ONBOARD_PROFILES_8100			(1 << 6)
 
 struct hidpp20drv_data {
 	struct hidpp20_device *dev;
@@ -61,10 +62,12 @@ struct hidpp20drv_data {
 	unsigned num_controls;
 	struct hidpp20_control_id *controls;
 	struct hidpp20_profiles *profiles;
+	struct hidpp20_color_led_zone_info *led_infos;
 
 	unsigned int num_profiles;
 	unsigned int num_resolutions;
 	unsigned int num_buttons;
+	unsigned int num_leds;
 };
 
 static void
@@ -225,6 +228,7 @@ hidpp20drv_read_button_8100(struct ratbag_button *button)
 		break;
 	default:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_UNKNOWN;
+		break;
 	}
 
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
@@ -240,6 +244,44 @@ hidpp20drv_read_button(struct ratbag_button *button)
 	hidpp20drv_read_button_8100(button);
 }
 
+static void
+hidpp20drv_read_led(struct ratbag_led *led)
+{
+	struct ratbag_device *device = led->profile->device;
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	struct hidpp20_profile *profile;
+	struct hidpp20_color_led_zone_info *led_info;
+	struct hidpp20_led *h_led;
+
+	if (!(drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070))
+		return;
+
+	led_info = &drv_data->led_infos[led->index];
+	profile = &drv_data->profiles->profiles[led->profile->index];
+	h_led = &profile->leds[led->index];
+
+	switch (h_led->mode) {
+	case HIDPP20_LED_ON:
+		led->mode = RATBAG_LED_ON;
+		break;
+	case HIDPP20_LED_CYCLE:
+		led->mode = RATBAG_LED_CYCLE;
+		break;
+	case HIDPP20_LED_BREATHING:
+		led->mode = RATBAG_LED_BREATHING;
+		break;
+	default:
+		led->mode = RATBAG_LED_OFF;
+		break;
+	}
+	led->type = hidpp20_8070_get_location_mapping(led_info->location);
+	led->color.red = h_led->color.red;
+	led->color.green = h_led->color.green;
+	led->color.blue = h_led->color.blue;
+	led->hz = h_led->period;  /* FIXME: should be ceil(1000.0 / h_led->period), but that would be very small */
+	led->brightness = h_led->brightness * 255 / 100;
+}
+
 static int
 hidpp20drv_write_button_1b04(struct ratbag_button *button,
 			     const struct ratbag_button_action *action)
@@ -276,8 +318,8 @@ hidpp20drv_write_button_1b04(struct ratbag_button *button,
 }
 
 static int
-hidpp20drv_write_button_8100(struct ratbag_button *button,
-			     const struct ratbag_button_action *action)
+hidpp20drv_update_button_8100(struct ratbag_button *button,
+			      const struct ratbag_button_action *action)
 {
 	struct ratbag_device *device = button->profile->device;
 	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
@@ -328,14 +370,14 @@ hidpp20drv_write_button_8100(struct ratbag_button *button,
 }
 
 static int
-hidpp20drv_write_button(struct ratbag_button *button,
+hidpp20drv_update_button(struct ratbag_button *button,
 			const struct ratbag_button_action *action)
 {
 	struct ratbag_device *device = button->profile->device;
 	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
 
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
-		return hidpp20drv_write_button_8100(button, action);
+		return hidpp20drv_update_button_8100(button, action);
 
 	if (drv_data->capabilities & HIDPP_CAP_BUTTON_KEY_1b04)
 		return hidpp20drv_write_button_1b04(button, action);
@@ -343,6 +385,48 @@ hidpp20drv_write_button(struct ratbag_button *button,
 	return -ENOTSUP;
 }
 
+static int
+hidpp20drv_update_led(struct ratbag_led *led)
+{
+	struct ratbag_profile *profile = led->profile;
+	struct ratbag_device *device = profile->device;
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	struct hidpp20_profile *h_profile;
+	struct hidpp20_led *h_led;
+
+	if (!(drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070))
+		return -ENOTSUP;
+
+	h_profile = &drv_data->profiles->profiles[profile->index];
+
+	h_led = &(h_profile->leds[led->index]);
+
+	if (!h_led)
+		return -EINVAL;
+
+	switch (led->mode) {
+	case RATBAG_LED_ON:
+		h_led->mode = HIDPP20_LED_ON;
+		break;
+	case RATBAG_LED_CYCLE:
+		h_led->mode = HIDPP20_LED_CYCLE;
+		break;
+	case RATBAG_LED_BREATHING:
+		h_led->mode = HIDPP20_LED_BREATHING;
+		break;
+	default:
+		h_led->mode = HIDPP20_LED_OFF;
+		break;
+	}
+	h_led->color.red = led->color.red;
+	h_led->color.green = led->color.green;
+	h_led->color.blue = led->color.blue;
+	h_led->period = led->hz; /* FIXME: should be 1000 / led->hz, but that would be very small */
+	h_led->brightness = led->brightness * 100 / 255;
+
+	return RATBAG_SUCCESS;
+}
+
 static int
 hidpp20drv_current_profile(struct ratbag_device *device)
 {
@@ -460,8 +544,8 @@ hidpp20drv_read_resolution_dpi(struct ratbag_profile *profile)
 }
 
 static int
-hidpp20drv_write_resolution_dpi_8100(struct ratbag_resolution *resolution,
-				     int dpi_x, int dpi_y)
+hidpp20drv_update_resolution_dpi_8100(struct ratbag_resolution *resolution,
+				      int dpi_x, int dpi_y)
 {
 	struct ratbag_profile *profile = resolution->profile;
 	struct ratbag_device *device = profile->device;
@@ -476,12 +560,12 @@ hidpp20drv_write_resolution_dpi_8100(struct ratbag_resolution *resolution,
 	h_profile = &drv_data->profiles->profiles[profile->index];
 	h_profile->dpi[index] = dpi;
 
-	return hidpp20_onboard_profiles_write(drv_data->dev, profile->index, drv_data->profiles);
+	return RATBAG_SUCCESS;
 }
 
 static int
-hidpp20drv_write_resolution_dpi(struct ratbag_resolution *resolution,
-				int dpi_x, int dpi_y)
+hidpp20drv_update_resolution_dpi(struct ratbag_resolution *resolution,
+				 int dpi_x, int dpi_y)
 {
 	struct ratbag_profile *profile = resolution->profile;
 	struct ratbag_device *device = profile->device;
@@ -491,7 +575,7 @@ hidpp20drv_write_resolution_dpi(struct ratbag_resolution *resolution,
 	int dpi = dpi_x; /* dpi_x == dpi_y if we don't have the individual resolution cap */
 
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
-		return hidpp20drv_write_resolution_dpi_8100(resolution, dpi_x, dpi_y);
+		return hidpp20drv_update_resolution_dpi_8100(resolution, dpi_x, dpi_y);
 
 	if (!(drv_data->capabilities & HIDPP_CAP_SWITCHABLE_RESOLUTION_2201))
 		return -ENOTSUP;
@@ -569,6 +653,29 @@ hidpp20drv_read_kbd_reprogrammable_key(struct ratbag_device *device)
 	return rc;
 }
 
+static int
+hidpp20drv_read_color_leds(struct ratbag_device *device)
+{
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	int rc;
+
+	if (!(drv_data->capabilities & HIDPP_CAP_COLOR_LED_EFFECTS_8070))
+		return 0;
+
+	free(drv_data->led_infos);
+	drv_data->led_infos = NULL;
+	drv_data->num_leds = 0;
+
+	rc = hidpp20_color_led_effects_get_zone_infos(drv_data->dev, &drv_data->led_infos);
+
+	if (rc > 0) {
+		drv_data->num_leds = rc;
+		rc = 0;
+	}
+
+	return rc;
+}
+
 static int
 hidpp20drv_read_onboard_profile(struct ratbag_device *device, unsigned index)
 {
@@ -637,12 +744,6 @@ hidpp20drv_read_profile(struct ratbag_profile *profile, unsigned int index)
 	profile->is_active = (index == 0);
 }
 
-static int
-hidpp20drv_write_profile(struct ratbag_profile *profile)
-{
-	return 0;
-}
-
 static int
 hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 {
@@ -718,6 +819,15 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 	}
 	case HIDPP_PAGE_COLOR_LED_EFFECTS: {
 		log_debug(ratbag, "device has color effects\n");
+		drv_data->capabilities |= HIDPP_CAP_COLOR_LED_EFFECTS_8070;
+
+		/* we read the profile once to get the correct number of
+		 * supported leds. */
+		if (hidpp20drv_read_color_leds(device))
+			return 0;
+
+		ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_LED);
+		device->num_leds = drv_data->num_leds;
 		break;
 	}
 	case HIDPP_PAGE_ONBOARD_PROFILES: {
@@ -731,6 +841,7 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		drv_data->num_profiles = drv_data->profiles->num_profiles;
 		drv_data->num_resolutions = drv_data->profiles->num_modes;
 		drv_data->num_buttons = drv_data->profiles->num_buttons;
+		drv_data->num_leds = drv_data->profiles->num_leds;
 
 		break;
 	}
@@ -744,6 +855,60 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 	return 0;
 }
 
+static int
+hidpp20_commit(struct ratbag_device *device)
+{
+	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	struct ratbag_profile *profile;
+	struct ratbag_button *button;
+	struct ratbag_led *led;
+	struct ratbag_resolution *resolution;
+	int rc;
+	unsigned int i;
+
+	list_for_each(profile, &device->profiles, link) {
+		if (!profile->dirty)
+			continue;
+
+		for (i = 0; i < profile->resolution.num_modes; i++) {
+			resolution = &profile->resolution.modes[i];
+
+			rc = hidpp20drv_update_resolution_dpi(resolution,
+							      resolution->dpi_x,
+							      resolution->dpi_y);
+			if (rc)
+				return RATBAG_ERROR_DEVICE;
+		}
+
+		list_for_each(button, &profile->buttons, link) {
+			struct ratbag_button_action action = button->action;
+
+			if (!button->dirty)
+				continue;
+
+			rc = hidpp20drv_update_button(button, &action);
+			if (rc)
+				return RATBAG_ERROR_DEVICE;
+		}
+
+		list_for_each(led, &profile->leds, link) {
+			if (!led->dirty)
+				continue;
+
+			rc = hidpp20drv_update_led(led);
+			if (rc)
+				return RATBAG_ERROR_DEVICE;
+		}
+
+		if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
+			hidpp20_onboard_profiles_write(drv_data->dev,
+						       profile->index,
+						       drv_data->profiles);
+	}
+
+	return RATBAG_SUCCESS;
+}
+
 static int
 hidpp20drv_20_probe(struct ratbag_device *device)
 {
@@ -788,12 +953,15 @@ hidpp20_log(void *userdata, enum hidpp_log_priority priority, const char *format
 static void
 hidpp20drv_remove(struct ratbag_device *device)
 {
-	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
-	struct hidpp20_device *dev = drv_data->dev;
+	struct hidpp20drv_data *drv_data;
+	struct hidpp20_device *dev;
 
 	if (!device)
 		return;
 
+	drv_data = ratbag_get_drv_data(device);
+	dev = drv_data->dev;
+
 	ratbag_close_hidraw(device);
 
 	if (drv_data->profiles)
@@ -859,6 +1027,7 @@ hidpp20drv_probe(struct ratbag_device *device)
 	drv_data->num_profiles = 1;
 	drv_data->num_resolutions = 1;
 	drv_data->num_buttons = 8;
+	drv_data->num_leds = 2;
 
 	rc = hidpp20drv_20_probe(device);
 	if (rc)
@@ -867,7 +1036,8 @@ hidpp20drv_probe(struct ratbag_device *device)
 	ratbag_device_init_profiles(device,
 				    drv_data->num_profiles,
 				    drv_data->num_resolutions,
-				    drv_data->num_buttons);
+				    drv_data->num_buttons,
+				    drv_data->num_leds);
 
 	return rc;
 err:
@@ -880,10 +1050,9 @@ struct ratbag_driver hidpp20_driver = {
 	.id = "hidpp20",
 	.probe = hidpp20drv_probe,
 	.remove = hidpp20drv_remove,
+	.commit = hidpp20_commit,
 	.read_profile = hidpp20drv_read_profile,
-	.write_profile = hidpp20drv_write_profile,
 	.set_active_profile = hidpp20drv_set_current_profile,
 	.read_button = hidpp20drv_read_button,
-	.write_button = hidpp20drv_write_button,
-	.write_resolution_dpi = hidpp20drv_write_resolution_dpi,
+	.read_led = hidpp20drv_read_led,
 };
