@@ -36,6 +36,7 @@
 #define ROCCAT_PROFILE_MAX			4
 #define ROCCAT_BUTTON_MAX			23
 #define ROCCAT_NUM_DPI				5
+#define ROCCAT_LED_MAX				0
 
 #define ROCCAT_MAX_RETRY_READY			10
 
@@ -302,6 +303,7 @@ roccat_wait_ready(struct ratbag_device *device)
 			return 2;
 
 		msleep(10);
+		count++;
 	}
 
 	return -ETIMEDOUT;
@@ -795,7 +797,8 @@ roccat_probe(struct ratbag_device *device)
 	ratbag_device_init_profiles(device,
 				    ROCCAT_PROFILE_MAX + 1,
 				    ROCCAT_NUM_DPI,
-				    ROCCAT_BUTTON_MAX + 1);
+				    ROCCAT_BUTTON_MAX + 1,
+				    ROCCAT_LED_MAX);
 
 	ratbag_device_set_capability(device, RATBAG_DEVICE_CAP_BUTTON_MACROS);
 
